@@ -1,5 +1,5 @@
 # This file is part of Pebble.
-# Copyright (c) 2013-2022, Matteo Cafasso
+# Copyright (c) 2013-2024, Matteo Cafasso
 
 # Pebble is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License
@@ -24,6 +24,7 @@ from typing import Callable, Optional
 from itertools import chain, count, islice
 from concurrent.futures import Future, TimeoutError
 
+from pebble.common import Result, SUCCESS
 from pebble.common import PebbleFuture, ProcessFuture, SLEEP_UNIT
 
 
@@ -220,10 +221,13 @@ class MapResults:
     def next(self):
         result = next(self._results)
 
-        if isinstance(result, Exception):
-            raise result
+        if isinstance(result, Result):
+            if result.status == SUCCESS:
+                return result.value
 
-        return result
+            result = result.value
+
+        raise result
 
     __next__ = next
 
@@ -262,7 +266,7 @@ def chunk_result(future: ProcessFuture, timeout: Optional[float]):
     """Returns the results of a processed chunk."""
     try:
         return future.result(timeout=timeout)
-    except Exception as error:
+    except BaseException as error:
         return (error, )
 
 
@@ -271,7 +275,7 @@ def run_initializer(initializer: Callable, initargs: list):
     try:
         initializer(*initargs)
         return True
-    except Exception as error:
+    except BaseException as error:
         logging.exception(error)
         return False
 
