@@ -1,5 +1,5 @@
 # This file is part of Pebble.
-# Copyright (c) 2013-2021, Matteo Cafasso
+# Copyright (c) 2013-2024, Matteo Cafasso
 
 # Pebble is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License
@@ -23,21 +23,19 @@ import multiprocessing
 from itertools import count
 from collections import namedtuple
 from signal import SIG_IGN, SIGINT, signal
+from typing import Any, Callable, Optional
+from concurrent.futures.process import BrokenProcessPool
 from concurrent.futures import CancelledError, TimeoutError
-try:
-    from concurrent.futures.process import BrokenProcessPool
-except ImportError:
-    class BrokenProcessPool(OSError):
-        pass
 
-from pebble.pool.channel import ChannelError, channels
-from pebble.pool.base_pool import BasePool, Task, TaskPayload
-from pebble.pool.base_pool import ProcessMapFuture, MapResults
-from pebble.pool.base_pool import iter_chunks, run_initializer
+from pebble.pool.base_pool import ProcessMapFuture, map_results
 from pebble.pool.base_pool import CREATED, ERROR, RUNNING, SLEEP_UNIT
+from pebble.pool.base_pool import Worker, iter_chunks, run_initializer
+from pebble.pool.base_pool import PoolContext, BasePool, Task, TaskPayload
+from pebble.pool.channel import ChannelError, WorkerChannel, channels
 from pebble.common import launch_process, stop_process
 from pebble.common import ProcessExpired, ProcessFuture
 from pebble.common import process_execute, launch_thread
+from pebble.common import Result, SUCCESS, FAILURE, ERROR as TASK_ERROR
 
 
 class ProcessPool(BasePool):
@@ -53,10 +51,12 @@ class ProcessPool(BasePool):
 
     """
 
-    def __init__(self, max_workers=multiprocessing.cpu_count(), max_tasks=0,
-                 initializer=None, initargs=(), context=None):
-        super(ProcessPool, self).__init__(
-            max_workers, max_tasks, initializer, initargs)
+    def __init__(self, max_workers: int = multiprocessing.cpu_count(),
+                 max_tasks: int = 0,
+                 initializer: Callable = None,
+                 initargs: list = (),
+                 context: multiprocessing.context.BaseContext = None):
+        super().__init__(max_workers, max_tasks, initializer, initargs)
         mp_context = multiprocessing if context is None else context
         self._pool_manager = PoolManager(self._context, mp_context)
         self._task_scheduler_loop = None
@@ -80,14 +80,16 @@ class ProcessPool(BasePool):
     def _stop_pool(self):
         if self._pool_manager_loop is not None:
             self._pool_manager_loop.join()
-        self._pool_manager.close()
+        self._pool_manager.stop()
         if self._task_scheduler_loop is not None:
             self._task_scheduler_loop.join()
         if self._message_manager_loop is not None:
             self._message_manager_loop.join()
-        self._pool_manager.stop()
 
-    def schedule(self, function, args=(), kwargs={}, timeout=None):
+    def schedule(self, function: Callable,
+                 args: list = (),
+                 kwargs: dict = {},
+                 timeout: float = None) -> ProcessFuture:
         """Schedules *function* to be run the Pool.
 
         *args* and *kwargs* will be forwareded to the scheduled function
@@ -109,7 +111,20 @@ class ProcessPool(BasePool):
 
         return future
 
-    def map(self, function, *iterables, **kwargs):
+    def submit(self, function: Callable,
+               timeout: Optional[float],
+               *args, **kwargs) -> ProcessFuture:
+        """This function is provided for compatibility with
+        `asyncio.loop.run_in_executor`.
+
+        For scheduling jobs within the pool use `schedule` instead.
+
+        """
+        return self.schedule(
+            function, args=args, kwargs=kwargs, timeout=timeout)
+
+    def map(self, function: Callable,
+            *iterables, **kwargs) -> ProcessMapFuture:
         """Computes the *function* using arguments from
         each of the iterables. Stops when the shortest iterable is exhausted.
 
@@ -135,28 +150,15 @@ class ProcessPool(BasePool):
             process_chunk, args=(function, chunk), timeout=timeout)
             for chunk in iter_chunks(chunksize, *iterables)]
 
-        map_future = ProcessMapFuture(futures)
-        if not futures:
-            map_future.set_result(MapResults(futures))
-            return map_future
-
-        def done_map(_):
-            if not map_future.done():
-                map_future.set_result(MapResults(futures, timeout=timeout))
-
-        for future in futures:
-            future.add_done_callback(done_map)
-            setattr(future, 'map_future', map_future)
-
-        return map_future
+        return map_results(ProcessMapFuture(futures), timeout)
 
 
-def task_scheduler_loop(pool_manager):
+def task_scheduler_loop(pool_manager: 'PoolManager'):
     context = pool_manager.context
     task_queue = context.task_queue
 
     try:
-        while context.alive and not global_shutdown:
+        while context.alive and not GLOBAL_SHUTDOWN:
             task = task_queue.get()
 
             if task is not None:
@@ -171,22 +173,22 @@ def task_scheduler_loop(pool_manager):
         context.state = ERROR
 
 
-def pool_manager_loop(pool_manager):
+def pool_manager_loop(pool_manager: 'PoolManager'):
     context = pool_manager.context
 
     try:
-        while context.alive and not global_shutdown:
+        while context.alive and not GLOBAL_SHUTDOWN:
             pool_manager.update_status()
             time.sleep(SLEEP_UNIT)
     except BrokenProcessPool:
         context.state = ERROR
 
 
-def message_manager_loop(pool_manager):
+def message_manager_loop(pool_manager: 'PoolManager'):
     context = pool_manager.context
 
     try:
-        while context.alive and not global_shutdown:
+        while context.alive and not GLOBAL_SHUTDOWN:
             pool_manager.process_next_message(SLEEP_UNIT)
     except BrokenProcessPool:
         context.state = ERROR
@@ -194,7 +196,8 @@ def message_manager_loop(pool_manager):
 
 class PoolManager:
     """Combines Task and Worker Managers providing a higher level one."""
-    def __init__(self, context, mp_context):
+    def __init__(self, context: PoolContext,
+                 mp_context: multiprocessing.context.BaseContext):
         self.context = context
         self.task_manager = TaskManager(context.task_queue.task_done)
         self.worker_manager = WorkerManager(context.workers,
@@ -204,13 +207,11 @@ class PoolManager:
     def start(self):
         self.worker_manager.create_workers()
 
-    def close(self):
-        self.worker_manager.close_channels()
-
     def stop(self):
+        self.worker_manager.close_channels()
         self.worker_manager.stop_workers()
 
-    def schedule(self, task):
+    def schedule(self, task: Task):
         """Schedules a new Task in the PoolManager."""
         self.task_manager.register(task)
         try:
@@ -218,15 +219,15 @@ class PoolManager:
         except (pickle.PicklingError, TypeError) as error:
             self.task_manager.task_problem(task.id, error)
 
-    def process_next_message(self, timeout):
+    def process_next_message(self, timeout: float):
         """Processes the next message coming from the workers."""
         message = self.worker_manager.receive(timeout)
 
         if isinstance(message, Acknowledgement):
             self.task_manager.task_start(message.task, message.worker)
-        elif isinstance(message, Result):
+        elif isinstance(message, TaskResult):
             self.task_manager.task_done(message.task, message.result)
-        elif isinstance(message, Problem):
+        elif isinstance(message, TaskProblem):
             self.task_manager.task_problem(message.task, message.error)
 
     def update_status(self):
@@ -236,14 +237,15 @@ class PoolManager:
     def update_tasks(self):
         """Handles timing out Tasks."""
         for task in self.task_manager.timeout_tasks():
-            self.task_manager.task_done(
-                task.id, TimeoutError("Task timeout", task.timeout))
             self.worker_manager.stop_worker(task.worker_id)
+            self.task_manager.task_done(
+                task.id,
+                Result(FAILURE, TimeoutError("Task timeout", task.timeout)))
 
         for task in self.task_manager.cancelled_tasks():
-            self.task_manager.task_done(
-                task.id, CancelledError())
             self.worker_manager.stop_worker(task.worker_id)
+            self.task_manager.task_done(
+                task.id, Result(FAILURE, CancelledError()))
 
     def update_workers(self):
         """Handles unexpected processes termination."""
@@ -252,7 +254,7 @@ class PoolManager:
 
         self.worker_manager.create_workers()
 
-    def handle_worker_expiration(self, expiration):
+    def handle_worker_expiration(self, expiration: tuple):
         worker_id, exitcode = expiration
 
         try:
@@ -261,16 +263,15 @@ class PoolManager:
             return
         else:
             error = ProcessExpired('Abnormal termination', code=exitcode)
-            self.task_manager.task_done(task.id, error)
+            self.task_manager.task_done(task.id, Result(TASK_ERROR, error))
 
-    def find_expired_task(self, worker_id):
-        tasks = tuple(self.task_manager.tasks.values())
+    def find_expired_task(self, worker_id: int) -> Task:
+        tasks = dictionary_values(self.task_manager.tasks)
         running_tasks = tuple(t for t in tasks if t.worker_id != 0)
-
         if running_tasks:
             return task_worker_lookup(running_tasks, worker_id)
-        else:
-            raise BrokenProcessPool("All workers expired")
+
+        raise BrokenProcessPool("All workers expired")
 
 
 class TaskManager:
@@ -280,20 +281,20 @@ class TaskManager:
     Timing out and cancelled tasks are handled as well.
     """
 
-    def __init__(self, task_done_callback):
+    def __init__(self, task_done_callback: Callable):
         self.tasks = {}
         self.task_done_callback = task_done_callback
 
-    def register(self, task):
+    def register(self, task: Task):
         self.tasks[task.id] = task
 
-    def task_start(self, task_id, worker_id):
+    def task_start(self, task_id: int, worker_id: Optional[int]):
         task = self.tasks[task_id]
         task.worker_id = worker_id
         task.timestamp = time.time()
         task.set_running_or_notify_cancel()
 
-    def task_done(self, task_id, result):
+    def task_done(self, task_id: int, result: Result):
         """Set the tasks result and run the callback."""
         try:
             task = self.tasks.pop(task_id)
@@ -302,27 +303,28 @@ class TaskManager:
         else:
             if task.future.cancelled():
                 task.set_running_or_notify_cancel()
-            elif isinstance(result, BaseException):
-                task.future.set_exception(result)
+            elif result.status == SUCCESS:
+                task.future.set_result(result.value)
             else:
-                task.future.set_result(result)
+                task.future.set_exception(result.value)
 
             self.task_done_callback()
 
-    def task_problem(self, task_id, error):
+    def task_problem(self, task_id: int, error: Exception):
         """Set the task with the error it caused within the Pool."""
         self.task_start(task_id, None)
-        self.task_done(task_id, error)
+        self.task_done(task_id, Result(TASK_ERROR, error))
 
-    def timeout_tasks(self):
-        return tuple(t for t in tuple(self.tasks.values()) if self.timeout(t))
+    def timeout_tasks(self) -> tuple:
+        return tuple(t for t in dictionary_values(self.tasks)
+                     if self.timeout(t))
 
-    def cancelled_tasks(self):
-        return tuple(t for t in tuple(self.tasks.values())
+    def cancelled_tasks(self) -> tuple:
+        return tuple(t for t in dictionary_values(self.tasks)
                      if t.timestamp != 0 and t.future.cancelled())
 
     @staticmethod
-    def timeout(task):
+    def timeout(task: Task) -> bool:
         if task.timeout and task.started:
             return time.time() - task.timestamp > task.timeout
         else:
@@ -335,45 +337,47 @@ class WorkerManager:
     Maintains the workers active and encapsulates their communication logic.
     """
 
-    def __init__(self, workers, worker_parameters, mp_context):
+    def __init__(self, workers:int,
+                 worker_parameters: Worker,
+                 mp_context: multiprocessing.context.BaseContext):
         self.workers = {}
         self.workers_number = workers
         self.worker_parameters = worker_parameters
         self.pool_channel, self.workers_channel = channels(mp_context)
         self.mp_context = mp_context
 
-    def dispatch(self, task):
+    def dispatch(self, task: Task):
         try:
             self.pool_channel.send(WorkerTask(task.id, task.payload))
         except (pickle.PicklingError, TypeError) as error:
             raise error
-        except (OSError, EnvironmentError, TypeError) as error:
-            raise BrokenProcessPool(error)
+        except OSError as error:
+            raise BrokenProcessPool from error
 
-    def receive(self, timeout):
+    def receive(self, timeout: float):
         try:
             if self.pool_channel.poll(timeout):
                 return self.pool_channel.recv()
             else:
                 return NoMessage()
-        except (OSError, EnvironmentError, TypeError) as error:
-            raise BrokenProcessPool(error)
+        except (OSError, TypeError) as error:
+            raise BrokenProcessPool from error
         except EOFError:  # Pool shutdown
             return NoMessage()
 
-    def inspect_workers(self):
+    def inspect_workers(self) -> tuple:
         """Updates the workers status.
 
         Returns the workers which have unexpectedly ended.
 
         """
-        workers = tuple(self.workers.values())
-        expired = tuple(w for w in workers if not w.is_alive())
+        expired = tuple(w for w in dictionary_values(self.workers)
+                        if not w.is_alive())
 
         for worker in expired:
             self.workers.pop(worker.pid)
 
-        return ((w.pid, w.exitcode) for w in expired if w.exitcode != 0)
+        return tuple((w.pid, w.exitcode) for w in expired if w.exitcode != 0)
 
     def create_workers(self):
         for _ in range(self.workers_number - len(self.workers)):
@@ -393,10 +397,10 @@ class WorkerManager:
                 WORKERS_NAME, worker_process, False, self.mp_context,
                 self.worker_parameters, self.workers_channel)
             self.workers[worker.pid] = worker
-        except (OSError, EnvironmentError) as error:
-            raise BrokenProcessPool(error)
+        except OSError as error:
+            raise BrokenProcessPool from error
 
-    def stop_worker(self, worker_id, force=False):
+    def stop_worker(self, worker_id: int, force=False):
         try:
             if force:
                 stop_process(self.workers.pop(worker_id))
@@ -404,12 +408,12 @@ class WorkerManager:
                 with self.workers_channel.lock:
                     stop_process(self.workers.pop(worker_id))
         except ChannelError as error:
-            raise BrokenProcessPool(error)
+            raise BrokenProcessPool from error
         except KeyError:
             return  # worker already expired
 
 
-def worker_process(params, channel):
+def worker_process(params: Worker, channel: WorkerChannel):
     """The worker process routines."""
     signal(SIGINT, SIG_IGN)
 
@@ -424,30 +428,30 @@ def worker_process(params, channel):
             payload = task.payload
             result = process_execute(
                 payload.function, *payload.args, **payload.kwargs)
-            send_result(channel, Result(task.id, result))
-    except (EnvironmentError, OSError, RuntimeError) as error:
+            send_result(channel, TaskResult(task.id, result))
+    except (OSError, RuntimeError) as error:
         errno = getattr(error, 'errno', 1)
         os._exit(errno if isinstance(errno, int) else 1)
     except EOFError as error:
         os._exit(0)
 
 
-def worker_get_next_task(channel, max_tasks):
+def worker_get_next_task(channel: WorkerChannel, max_tasks: int):
     counter = count()
 
     while max_tasks == 0 or next(counter) < max_tasks:
         yield fetch_task(channel)
 
 
-def send_result(pipe, result):
+def send_result(channel: WorkerChannel, result: Any):
     """Send result handling pickling and communication errors."""
     try:
-        pipe.send(result)
+        channel.send(result)
     except (pickle.PicklingError, TypeError) as error:
-        pipe.send(Problem(result.task, error))
+        channel.send(TaskProblem(result.task, error))
 
 
-def fetch_task(channel):
+def fetch_task(channel: WorkerChannel) -> Task:
     while channel.poll():
         try:
             return task_transaction(channel)
@@ -455,7 +459,7 @@ def fetch_task(channel):
             continue  # another worker got the task
 
 
-def task_transaction(channel):
+def task_transaction(channel: WorkerChannel) -> Task:
     """Ensures a task is fetched and acknowledged atomically."""
     with channel.lock:
         if channel.poll(0):
@@ -467,7 +471,7 @@ def task_transaction(channel):
     return task
 
 
-def task_worker_lookup(running_tasks, worker_id):
+def task_worker_lookup(running_tasks: tuple, worker_id: int) -> Task:
     for task in running_tasks:
         if task.worker_id == worker_id:
             return task
@@ -475,14 +479,14 @@ def task_worker_lookup(running_tasks, worker_id):
     raise LookupError("Not found")
 
 
-def process_chunk(function, chunk):
+def process_chunk(function: Callable, chunk: list) -> list:
     """Processes a chunk of the iterable passed to map dealing with errors."""
     return [process_execute(function, *args) for args in chunk]
 
 
 def interpreter_shutdown():
-    global global_shutdown
-    global_shutdown = True
+    global GLOBAL_SHUTDOWN
+    GLOBAL_SHUTDOWN = True
 
     workers = [p for p in multiprocessing.active_children()
                if p.name == WORKERS_NAME]
@@ -491,13 +495,22 @@ def interpreter_shutdown():
         stop_process(worker)
 
 
+def dictionary_values(dictionary: dict) -> tuple:
+    """Returns a snapshot of the dictionary values handling race conditions."""
+    while True:
+        try:
+            return tuple(dictionary.values())
+        except RuntimeError:  # race condition
+            pass
+
+
 atexit.register(interpreter_shutdown)
 
 
-global_shutdown = False
+GLOBAL_SHUTDOWN = False
 WORKERS_NAME = 'pebble_pool_worker'
 NoMessage = namedtuple('NoMessage', ())
-Result = namedtuple('Result', ('task', 'result'))
-Problem = namedtuple('Problem', ('task', 'error'))
+TaskResult = namedtuple('TaskResult', ('task', 'result'))
+TaskProblem = namedtuple('TaskProblem', ('task', 'error'))
 WorkerTask = namedtuple('WorkerTask', ('id', 'payload'))
 Acknowledgement = namedtuple('Acknowledgement', ('worker', 'task'))
