@@ -6,6 +6,7 @@ import signal
 import asyncio
 import unittest
 import threading
+import dataclasses
 import multiprocessing
 
 from concurrent.futures import CancelledError, TimeoutError
@@ -44,7 +45,7 @@ def long_initializer():
 
 
 def broken_initializer():
-    raise Exception("BOOM!")
+    raise BaseException("BOOM!")
 
 
 def function(argument, keyword_argument=0):
@@ -57,7 +58,20 @@ def initializer_function():
 
 
 def error_function():
-    raise Exception("BOOM!")
+    raise BaseException("BOOM!")
+
+
+def return_error_function():
+    return BaseException("BOOM!")
+
+
+@dataclasses.dataclass(frozen=True)
+class FrozenError(Exception):
+    pass
+
+
+def frozen_error_function():
+    raise FrozenError()
 
 
 def pickle_error_function():
@@ -120,7 +134,7 @@ class TestProcessPool(unittest.TestCase):
     def callback(self, future):
         try:
             self.result = future.result()
-        except Exception as error:
+        except BaseException as error:
             self.exception = error
         finally:
             self.event.set()
@@ -153,7 +167,13 @@ class TestProcessPool(unittest.TestCase):
         """Process Pool Fork errors are raised by future get."""
         with ProcessPool(max_workers=1, context=mp_context) as pool:
             future = pool.schedule(error_function)
-        self.assertRaises(Exception, future.result)
+        self.assertRaises(BaseException, future.result)
+
+    def test_process_pool_error_returned(self):
+        """Process Pool Fork returned errors are returned by future get."""
+        with ProcessPool(max_workers=1, context=mp_context) as pool:
+            future = pool.schedule(return_error_function)
+        self.assertIsInstance(future.result(), BaseException)
 
     def test_process_pool_error_callback(self):
         """Process Pool Fork errors are forwarded to callback."""
@@ -161,7 +181,7 @@ class TestProcessPool(unittest.TestCase):
             future = pool.schedule(error_function)
         future.add_done_callback(self.callback)
         self.event.wait()
-        self.assertTrue(isinstance(self.exception, Exception))
+        self.assertTrue(isinstance(self.exception, BaseException))
 
     def test_process_pool_pickling_error_task(self):
         """Process Pool Fork task pickling errors
@@ -177,6 +197,12 @@ class TestProcessPool(unittest.TestCase):
             future = pool.schedule(pickle_error_function)
             self.assertRaises((pickle.PicklingError, TypeError), future.result)
 
+    def test_process_pool_frozen_error(self):
+        """Process Pool Fork frozen errors are raised by future get."""
+        with ProcessPool(max_workers=1, context=mp_context) as pool:
+            future = pool.schedule(frozen_error_function)
+        self.assertRaises(FrozenError, future.result)
+
     def test_process_pool_timeout(self):
         """Process Pool Fork future raises TimeoutError if so."""
         with ProcessPool(max_workers=1, context=mp_context) as pool:
@@ -378,12 +404,12 @@ class TestProcessPool(unittest.TestCase):
                           kwargs={'keyword_argument': 1})
 
     def test_process_pool_exception_isolated(self):
-        """Process Pool Fork an Exception does not affect other futures."""
+        """Process Pool Fork an BaseException does not affect other futures."""
         with ProcessPool(max_workers=1, context=mp_context) as pool:
             future = pool.schedule(error_function)
             try:
                 future.result()
-            except Exception:
+            except BaseException:
                 pass
             future = pool.schedule(function, args=[1],
                                    kwargs={'keyword_argument': 1})
@@ -563,7 +589,7 @@ class TestAsyncIOProcessPool(unittest.TestCase):
     def callback(self, future):
         try:
             self.result = future.result()
-        # asyncio.exception.CancelledError does not inherit from Exception
+        # asyncio.exception.CancelledError does not inherit from BaseException
         except BaseException as error:
             self.exception = error
         finally:
@@ -619,9 +645,19 @@ class TestAsyncIOProcessPool(unittest.TestCase):
             return await loop.run_in_executor(pool, error_function, None)
 
         with ProcessPool(max_workers=1, context=mp_context) as pool:
-            with self.assertRaises(Exception):
+            with self.assertRaises(BaseException):
                 asyncio.run(test(pool))
 
+    def test_process_pool_error_returned(self):
+        """Process Pool Fork returned errors are returned by future get."""
+        async def test(pool):
+            loop = asyncio.get_running_loop()
+
+            return await loop.run_in_executor(pool, return_error_function, None)
+
+        with ProcessPool(max_workers=1, context=mp_context) as pool:
+            self.assertIsInstance(asyncio.run(test(pool)), BaseException)
+
     def test_process_pool_error_callback(self):
         """Process Pool Fork errors are forwarded to callback."""
         async def test(pool):
@@ -637,7 +673,7 @@ class TestAsyncIOProcessPool(unittest.TestCase):
 
         with ProcessPool(max_workers=1, context=mp_context) as pool:
             asyncio.run(test(pool))
-            self.assertTrue(isinstance(self.exception, Exception))
+            self.assertTrue(isinstance(self.exception, BaseException))
 
     def test_process_pool_timeout(self):
         """Process Pool Fork future raises TimeoutError if so."""
