@@ -6,6 +6,7 @@ import signal
 import asyncio
 import unittest
 import threading
+import dataclasses
 import multiprocessing
 
 from concurrent.futures import CancelledError, TimeoutError
@@ -44,7 +45,7 @@ def long_initializer():
 
 
 def broken_initializer():
-    raise Exception("BOOM!")
+    raise BaseException("BOOM!")
 
 
 def function(argument, keyword_argument=0):
@@ -57,13 +58,26 @@ def initializer_function():
 
 
 def error_function():
-    raise Exception("BOOM!")
+    raise BaseException("BOOM!")
+
+
+def return_error_function():
+    return BaseException("BOOM!")
 
 
 def pickle_error_function():
     return threading.Lock()
 
 
+@dataclasses.dataclass(frozen=True)
+class FrozenError(Exception):
+    pass
+
+
+def frozen_error_function():
+    raise FrozenError()
+
+
 def long_function(value=1):
     time.sleep(value)
     return value
@@ -120,7 +134,7 @@ class TestProcessPool(unittest.TestCase):
     def callback(self, future):
         try:
             self.result = future.result()
-        except Exception as error:
+        except BaseException as error:
             self.exception = error
         finally:
             self.event.set()
@@ -153,7 +167,13 @@ class TestProcessPool(unittest.TestCase):
         """Process Pool Spawn errors are raised by future get."""
         with ProcessPool(max_workers=1, context=mp_context) as pool:
             future = pool.schedule(error_function)
-        self.assertRaises(Exception, future.result)
+        self.assertRaises(BaseException, future.result)
+
+    def test_process_pool_error_returned(self):
+        """Process Pool Spawn returned errors are returned by future get."""
+        with ProcessPool(max_workers=1, context=mp_context) as pool:
+            future = pool.schedule(return_error_function)
+        self.assertIsInstance(future.result(), BaseException)
 
     def test_process_pool_error_callback(self):
         """Process Pool Spawn errors are forwarded to callback."""
@@ -161,7 +181,7 @@ class TestProcessPool(unittest.TestCase):
             future = pool.schedule(error_function)
         future.add_done_callback(self.callback)
         self.event.wait()
-        self.assertTrue(isinstance(self.exception, Exception))
+        self.assertTrue(isinstance(self.exception, BaseException))
 
     def test_process_pool_pickling_error_task(self):
         """Process Pool Spawn task pickling errors
@@ -177,6 +197,12 @@ class TestProcessPool(unittest.TestCase):
             future = pool.schedule(pickle_error_function)
             self.assertRaises((pickle.PicklingError, TypeError), future.result)
 
+    def test_process_pool_frozen_error(self):
+        """Process Pool Spawn frozen errors are raised by future get."""
+        with ProcessPool(max_workers=1, context=mp_context) as pool:
+            future = pool.schedule(frozen_error_function)
+        self.assertRaises(FrozenError, future.result)
+
     def test_process_pool_timeout(self):
         """Process Pool Spawn future raises TimeoutError if so."""
         with ProcessPool(max_workers=1, context=mp_context) as pool:
@@ -315,7 +341,7 @@ class TestProcessPool(unittest.TestCase):
             future = pool.schedule(function, args=[1])
             future.add_done_callback(stop_pool_callback)
             with self.assertRaises(RuntimeError):
-                for index in range(10):
+                for index in range(30):
                     time.sleep(0.1)
                     pool.schedule(long_function, args=[index])
 
@@ -378,12 +404,12 @@ class TestProcessPool(unittest.TestCase):
                           kwargs={'keyword_argument': 1})
 
     def test_process_pool_exception_isolated(self):
-        """Process Pool Spawn an Exception does not affect other futures."""
+        """Process Pool Spawn an BaseException does not affect other futures."""
         with ProcessPool(max_workers=1, context=mp_context) as pool:
             future = pool.schedule(error_function)
             try:
                 future.result()
-            except Exception:
+            except BaseException:
                 pass
             future = pool.schedule(function, args=[1],
                                    kwargs={'keyword_argument': 1})
@@ -562,14 +588,14 @@ class TestAsyncIOProcessPool(unittest.TestCase):
     def callback(self, future):
         try:
             self.result = future.result()
-        # asyncio.exception.CancelledError does not inherit from Exception
+        # asyncio.exception.CancelledError does not inherit from BaseException
         except BaseException as error:
             self.exception = error
         finally:
             self.event.set()
 
     def test_process_pool_single_future(self):
-        """Process Pool Fork single future."""
+        """Process Pool Spawn single future."""
         async def test(pool):
             loop = asyncio.get_running_loop()
 
@@ -579,7 +605,7 @@ class TestAsyncIOProcessPool(unittest.TestCase):
             self.assertEqual(asyncio.run(test(pool)), 1)
 
     def test_process_pool_multiple_futures(self):
-        """Process Pool Fork multiple futures."""
+        """Process Pool Spawn multiple futures."""
         async def test(pool):
             futures = []
             loop = asyncio.get_running_loop()
@@ -594,7 +620,7 @@ class TestAsyncIOProcessPool(unittest.TestCase):
                                  for r in asyncio.run(test(pool))[0]), 5)
 
     def test_process_pool_callback(self):
-        """Process Pool Fork result is forwarded to the callback."""
+        """Process Pool Spawn result is forwarded to the callback."""
         async def test(pool):
             loop = asyncio.get_running_loop()
 
@@ -611,18 +637,28 @@ class TestAsyncIOProcessPool(unittest.TestCase):
             self.assertEqual(self.result, 1)
 
     def test_process_pool_error(self):
-        """Process Pool Fork errors are raised by future get."""
+        """Process Pool Spawn errors are raised by future get."""
         async def test(pool):
             loop = asyncio.get_running_loop()
 
             return await loop.run_in_executor(pool, error_function, None)
 
         with ProcessPool(max_workers=1, context=mp_context) as pool:
-            with self.assertRaises(Exception):
+            with self.assertRaises(BaseException):
                 asyncio.run(test(pool))
 
+    def test_process_pool_error_returned(self):
+        """Process Pool Spawn returned errors are returned by future get."""
+        async def test(pool):
+            loop = asyncio.get_running_loop()
+
+            return await loop.run_in_executor(pool, return_error_function, None)
+
+        with ProcessPool(max_workers=1, context=mp_context) as pool:
+            self.assertIsInstance(asyncio.run(test(pool)), BaseException)
+
     def test_process_pool_error_callback(self):
-        """Process Pool Fork errors are forwarded to callback."""
+        """Process Pool Spawn errors are forwarded to callback."""
         async def test(pool):
             loop = asyncio.get_running_loop()
 
@@ -636,10 +672,10 @@ class TestAsyncIOProcessPool(unittest.TestCase):
 
         with ProcessPool(max_workers=1, context=mp_context) as pool:
             asyncio.run(test(pool))
-            self.assertTrue(isinstance(self.exception, Exception))
+            self.assertTrue(isinstance(self.exception, BaseException))
 
     def test_process_pool_timeout(self):
-        """Process Pool Fork future raises TimeoutError if so."""
+        """Process Pool Spawn future raises TimeoutError if so."""
         async def test(pool):
             loop = asyncio.get_running_loop()
 
@@ -650,7 +686,7 @@ class TestAsyncIOProcessPool(unittest.TestCase):
                 asyncio.run(test(pool))
 
     def test_process_pool_timeout_callback(self):
-        """Process Pool Fork TimeoutError is forwarded to callback."""
+        """Process Pool Spawn TimeoutError is forwarded to callback."""
         async def test(pool):
             loop = asyncio.get_running_loop()
 
@@ -669,7 +705,7 @@ class TestAsyncIOProcessPool(unittest.TestCase):
             self.assertTrue(isinstance(self.exception, asyncio.TimeoutError))
 
     def test_process_pool_cancel(self):
-        """Process Pool Fork future raises CancelledError if so."""
+        """Process Pool Spawn future raises CancelledError if so."""
         async def test(pool):
             loop = asyncio.get_running_loop()
             future = loop.run_in_executor(pool, long_function, None)
@@ -685,7 +721,7 @@ class TestAsyncIOProcessPool(unittest.TestCase):
                 asyncio.run(test(pool))
 
     def test_process_pool_cancel_callback(self):
-        """Process Pool Fork CancelledError is forwarded to callback."""
+        """Process Pool Spawn CancelledError is forwarded to callback."""
         async def test(pool):
             loop = asyncio.get_running_loop()
 
@@ -706,7 +742,7 @@ class TestAsyncIOProcessPool(unittest.TestCase):
             self.assertTrue(isinstance(self.exception, asyncio.CancelledError))
 
     def test_process_pool_stop_timeout(self):
-        """Process Pool Fork workers are stopped if future timeout."""
+        """Process Pool Spawn workers are stopped if future timeout."""
         async def test(pool):
             loop = asyncio.get_running_loop()
             future1 = loop.run_in_executor(pool, pid_function, None)
@@ -720,7 +756,7 @@ class TestAsyncIOProcessPool(unittest.TestCase):
             asyncio.run(test(pool))
 
     def test_process_pool_stop_cancel(self):
-        """Process Pool Fork workers are stopped if future is cancelled."""
+        """Process Pool Spawn workers are stopped if future is cancelled."""
         async def test(pool):
             loop = asyncio.get_running_loop()
             future1 = loop.run_in_executor(pool, pid_function, None)
