@@ -2,6 +2,7 @@ import time
 import asyncio
 import unittest
 import threading
+import dataclasses
 
 from pebble import ThreadPool
 
@@ -13,7 +14,7 @@ initarg = 0
 
 
 def error_callback(future):
-    raise Exception("BOOM!")
+    raise BaseException("BOOM!")
 
 
 def initializer(value):
@@ -22,7 +23,7 @@ def initializer(value):
 
 
 def broken_initializer():
-    raise Exception("BOOM!")
+    raise BaseException("BOOM!")
 
 
 def function(argument, keyword_argument=0):
@@ -35,7 +36,16 @@ def initializer_function():
 
 
 def error_function():
-    raise Exception("BOOM!")
+    raise BaseException("BOOM!")
+
+
+@dataclasses.dataclass(frozen=True)
+class FrozenError(Exception):
+    pass
+
+
+def frozen_error_function():
+    raise FrozenError()
 
 
 def long_function(value=0):
@@ -60,7 +70,7 @@ class TestThreadPool(unittest.TestCase):
     def callback(self, future):
         try:
             self.results = future.result()
-        except Exception as error:
+        except BaseException as error:
             self.exception = error
         finally:
             self.event.set()
@@ -94,7 +104,7 @@ class TestThreadPool(unittest.TestCase):
         """Thread Pool errors are raised by future get."""
         with ThreadPool(max_workers=1) as pool:
             future = pool.schedule(error_function)
-        with self.assertRaises(Exception):
+        with self.assertRaises(BaseException):
             future.result()
 
     def test_thread_pool_error_callback(self):
@@ -103,7 +113,13 @@ class TestThreadPool(unittest.TestCase):
             future = pool.schedule(error_function)
             future.add_done_callback(self.callback)
         self.event.wait()
-        self.assertTrue(isinstance(self.exception, Exception))
+        self.assertTrue(isinstance(self.exception, BaseException))
+
+    def test_process_pool_frozen_error(self):
+        """Thread Pool frozen errors are raised by future get."""
+        with ThreadPool(max_workers=1) as pool:
+            future = pool.schedule(frozen_error_function)
+        self.assertRaises(FrozenError, future.result)
 
     def test_thread_pool_cancel_callback(self):
         """Thread Pool FutureCancelled is forwarded to callback."""
@@ -244,7 +260,7 @@ class TestThreadPool(unittest.TestCase):
         pool.join()
 
     def test_thread_pool_exception_isolated(self):
-        """Thread Pool an Exception does not affect other futures."""
+        """Thread Pool an BaseException does not affect other futures."""
         with ThreadPool(max_workers=1) as pool:
             future = pool.schedule(error_function)
             try:
@@ -374,7 +390,7 @@ class TestAsyncIOThreadPool(unittest.TestCase):
     def callback(self, future):
         try:
             self.result = future.result()
-        # asyncio.exception.CancelledError does not inherit from Exception
+        # asyncio.exception.CancelledError does not inherit from BaseException
         except BaseException as error:
             self.exception = error
         finally:
@@ -430,7 +446,7 @@ class TestAsyncIOThreadPool(unittest.TestCase):
             return await loop.run_in_executor(pool, error_function)
 
         with ThreadPool(max_workers=1) as pool:
-            with self.assertRaises(Exception):
+            with self.assertRaises(BaseException):
                 asyncio.run(test(pool))
 
     def test_thread_pool_error_callback(self):
@@ -448,7 +464,7 @@ class TestAsyncIOThreadPool(unittest.TestCase):
 
         with ThreadPool(max_workers=1) as pool:
             asyncio.run(test(pool))
-            self.assertTrue(isinstance(self.exception, Exception))
+            self.assertTrue(isinstance(self.exception, BaseException))
 
     def test_thread_pool_cancel_callback(self):
         """Thread Pool FutureCancelled is forwarded to callback."""
